MindDrift.Game = function(game) {
    this.platformGroup;
    this.player;
    this.badMemoriesGroup;
    this.neuroTransmittorsGroup;    
    this.playableBounds;
    this.door;
    this.collisionGroups = {};
    this.hud = {};
    this.gameMusic;
};

MindDrift.Game.prototype = {    
    create:function() {
        //init
        this.physics.startSystem(Phaser.Physics.P2JS);
        this.physics.p2.setImpactEvents(true);
        this.physics.p2.restitution = 0.001;
        this.physics.p2.gravity.y = 300;
        this.world.setBounds(0, 0, this.world.width * 5, this.world.height * 1);
        this.add.tileSprite(0, 0, this.world.width, this.world.height, 'background');
        
        //need to create collision groups before any objects apparently
        this.collisionGroups.player = this.physics.p2.createCollisionGroup();
        this.collisionGroups.door = this.physics.p2.createCollisionGroup();        
        this.collisionGroups.neuroTransmittor = this.physics.p2.createCollisionGroup();
        this.collisionGroups.badMemories = this.physics.p2.createCollisionGroup();
        this.collisionGroups.platforms = this.physics.p2.createCollisionGroup();

        this.buildDoor();
        this.buildWorld();
        this.playableBounds = this.getPlayableBounds();
        this.buildBadMemories();
        this.buildNeurotransmittors();

        this.physics.p2.restitution = 0.001;        
        this.physics.p2.gravity.y = 300;
        
        // Get the player
        this.buildPlayer();

        //controls
        this.cursors = this.input.keyboard.createCursorKeys();
        
        //HUD elements last so they are always on top
        this.buildHud();
        
        //wire up collision detection on the groups
        this.player.body.collides(this.collisionGroups.neuroTransmittor, this.playerNeuroTransmittorCollide, this);
        this.player.body.collides(this.collisionGroups.badMemories, this.playerBadMemoriesCollide, this);
        this.player.body.collides(this.collisionGroups.platforms, null, this);
        this.player.body.collides(this.collisionGroups.door, this.playerDoorCollide, this);
        
        //dafuq, this seems hacky but how colliding with world boundries works, call after each sprite you want to collide with world
        //http://www.html5gamedevs.com/topic/16824-p2-physics-how-to-make-a-player-collide-with-world-bounds-but-not-enemies/
        this.physics.p2.updateBoundsCollisionGroup();
        
        this.gameMusic = this.add.audio('game-music');
        this.gameMusic.play('', 0, 0.3, true);
    },
    
    shutdown:function() {
        this.gameMusic.stop();
        this.world.setBounds(0, 0, 800, 600);
    },
    
    update:function() {
        
        this.updateHud();

        // Default to player not moving
        this.player.body.velocity.x = 0;
        
        // Move character with arrow keys
        if (this.cursors.left.isDown)
        {   //Move to the left
            this.player.scale.setTo(-1, 1);
            this.player.body.velocity.x = -415;
            this.player.animations.play('walk');
        }

        //Move to the right
        else if (this.cursors.right.isDown)
       {   
            this.player.scale.setTo(1, 1);
            this.player.body.velocity.x = 415;
            this.player.animations.play('walk');
       } 

        //Stand still
        else
         {  
            this.player.animations.stop();
            this.player.frame = 0;
        }
        

        // Jump enabled if touching the ground
        if (this.cursors.up.isDown && this.isTouchingGroup(this.player))
        {
            this.player.body.velocity.y = -450;
        }

        // Play animations for groups
        this.neuroTransmittorsGroup.callAll('play', null, 'rotate');
        this.darkSmokesGroup.callAll('play', null, 'dShimmer');
        this.badMemoriesGroup.callAll('play', null, 'menace1');       
    },
    
    buildHud: function() {
        
        //background hud
        this.hud.background = this.add.tileSprite(20, 20, 300, 150, 'off-white-bg');
        this.hud.background.alpha = 0.4;
        this.hud.background.fixedToCamera = true;
        
        //progress bar
        this.hud.progressBar = {
            fillPercentage: .6,
            x: 60,
            y: 55,
            maxWidth: 200,
            height: 30,
        };
        
        var fontText = 18;
        this.hud.barText = this.add.bitmapText(110, 25, 'game-font', 'Sanity Bar', fontText);
        this.hud.barText.fixedToCamera = true;
        
        this.hud.progressBar.backgroundSprite = this.add.tileSprite(this.hud.progressBar.x, this.hud.progressBar.y, this.hud.progressBar.maxWidth, this.hud.progressBar.height, 'progressBarBackground');
        this.hud.progressBar.backgroundSprite.fixedToCamera = true;
        
        this.hud.progressBar.foregroundSprite = this.add.tileSprite(this.hud.progressBar.x, this.hud.progressBar.y, this.hud.progressBar.maxWidth, this.hud.progressBar.height, 'progressBarForeground');
        this.hud.progressBar.foregroundSprite.fixedToCamera = true;

        //score text
        this.hud.numberOfGoodMemories = 0;
        this.hud.scoreText = this.add.bitmapText(25, 105, 'game-font', '', fontText);
        this.hud.scoreText.fixedToCamera = true;
       // this.hud.scoreText.anchor.setTo(0.5);
        
        //flashing "got hurt" overlay
        this.hud.bloodOverlay = this.add.tileSprite(0, 0, this.world.width, this.world.height, 'blood-swatch');
        this.hud.bloodOverlay.alpha = 0;
        
        //flashing "got healed" overlay
        this.hud.healOverlay = this.add.tileSprite(0, 0, this.world.width, this.world.height, 'heal-swatch');
        this.hud.healOverlay.alpha = 0;
    },
    
    isTouchingGroup:function(someone) {
        var yAxis = p2.vec2.fromValues(0, 1);
        var result = false;
        for (var i = 0; i < this.physics.p2.world.narrowphase.contactEquations.length; i++) 
        {        
            var c = this.physics.p2.world.narrowphase.contactEquations[i];  // cycles through all the contactEquations until it finds our "someone"
            if (c.bodyA === someone.body.data || c.bodyB === someone.body.data) {
                var d = p2.vec2.dot(c.normalA, yAxis); // Normal dot Y-axis
                if (c.bodyA === someone.body.data) d *= -1;
                if (d > 0.5) result = true;
            }
        }
        return result;
    },
    
    playerNeuroTransmittorCollide:function(b1, b2) {
        
        //something is probably wrong in my code, every collision seems to come in twice
        if (!b1.hasCollided)
        {
            //play audio
            this.add.audio('good-memory-collide').play();

            //decrease health
            var percentHealed = 0.05;

            if (this.hud.progressBar.fillPercentage + percentHealed >= 1) {
                this.hud.progressBar.fillPercentage = 1;
                //max score bar or whatever;
            }
            else{
                this.hud.progressBar.fillPercentage += percentHealed;
            }
            
            //flash health
            this.hud.healOverlay.alpha = .5;
            
            //increment, getting collision events twice here and not sure why
            this.hud.numberOfGoodMemories++;
            b1.hasCollided = true;
            
            //kill this object
            b2.sprite.kill();
        }
        else
        {
            b1.hasCollided = false;
        }
    },
    
    
    playerDoorCollide:function(b1, b2) {
        this.state.start('WonGameMenu');
    },    
    

    // Build end door - no ewoks plz
    buildDoor:function() {
        
        var bounds = this.getPlayableBounds();
        this.door = this.add.sprite(bounds.end.x + 200, 330, 'door');
        this.door.anchor.setTo(0.5, 0.5);        
        this.physics.p2.enable(this.door);
        this.door.body.static = true; 
        this.door.body.setCollisionGroup(this.collisionGroups.door);
        this.door.body.collides(this.collisionGroups.player);
    },
    
    
    playerBadMemoriesCollide:function(b1, b2) {
        
        //something is probably wrong in my code, every collision seems to come in twice
        if (!b1.hasCollided)
        {
            //play audio
            this.add.audio('bad-memory-collide').play();
        
            //decrease health
            var percentHurt = 0.05;

            if (this.hud.progressBar.fillPercentage - percentHurt <= 0) {
                this.hud.progressBar.fillPercentage = 0;
                
                //dead
                this.state.start('LostGameMenu');
            }
            else{
                this.hud.progressBar.fillPercentage -= percentHurt;
            }
            
            this.hud.bloodOverlay.alpha = .5;
        }
        else
        {
            b1.hasCollided = true;
        }
    },
    
    buildBadMemories:function() {
        this.darkSmokesGroup = this.add.group();
        this.darkSmokesGroup.enableBody = true;
        this.darkSmokesGroup.physicsBodyType = Phaser.Physics.P2JS;
        this.badMemoriesGroup = this.add.group();
        this.badMemoriesGroup.enableBody = true;
        this.badMemoriesGroup.physicsBodyType = Phaser.Physics.P2JS;
        
        var numBadMemories = 15;
        for (var i = 0; i<numBadMemories; i++)
        {
            var location = this.getRandomCoordinatesInPlayableBounds();
            var dSmoke = this.darkSmokesGroup.create(location.x, location.y, 'darkSmoke');
            var badGuy = this.badMemoriesGroup.create(location.x, location.y, 'grue');
            dSmoke.body.static = true;
            dSmoke.scale.set(2.4);
            badGuy.scale.set(.8);
            badGuy.alpha = 0.4;
            badGuy.body.setRectangle(badGuy.width, badGuy.height);
            badGuy.body.static = true;
            badGuy.body.setCollisionGroup(this.collisionGroups.badMemories);
            badGuy.body.collides(this.collisionGroups.player);
        }
        
        this.badMemoriesGroup.callAll('animations.add', 'animations', 'menace1', [3, 0, 3, 2], 7, true);
        this.darkSmokesGroup.callAll('animations.add', 'animations', 'dShimmer', [0, 1, 2, 3, 4, 5, 6, 5, 4, 3, 2, 1], 10, true);
    },
    
    buildNeurotransmittors:function() {

        this.neuroTransmittorsGroup = this.add.group();
        this.neuroTransmittorsGroup.enableBody = true;
        this.neuroTransmittorsGroup.physicsBodyType = Phaser.Physics.P2JS;
        
        var numNeuroTransmittors = 10;
        for (var i = 0; i<numNeuroTransmittors; i++)
        {
            var location = this.getRandomCoordinatesInPlayableBounds();
            var nt = this.neuroTransmittorsGroup.create(location.x, location.y, 'blueOrb');
            nt.scale.set(.7);
            nt.body.static = true
            nt.body.setCollisionGroup(this.collisionGroups.neuroTransmittor);
            nt.body.collides(this.collisionGroups.player);
        }
        
        // Create animation for group
        this.neuroTransmittorsGroup.callAll('animations.add', 'animations', 'rotate', [0, 1, 2, 3], 10, true);
    },
    
    //give ourselves some buffer space so we don't spawn in floor/ceiling/beginning/end areas
    getPlayableBounds: function() {
        var bounds = {
            buffer: {
                x: 400,
                y: 100,
            },
        };
        
        bounds.start = {
            x: bounds.buffer.x,
            y: bounds.buffer.y,
        };
        
        bounds.end = {
            x: this.world.width - bounds.buffer.x,
            y: this.world.height - bounds.buffer.y,
        };
        
        return bounds;
    },
    
    getRandomCoordinatesInPlayableBounds:function() {
        var bounds = this.playableBounds;
        var x = this.rnd.integerInRange(bounds.start.x, bounds.end.x);
        var y = this.rnd.integerInRange(bounds.start.y, bounds.end.y);
        
//        //leave this here for now, close to working but needs more time
//        if (this.hasObjectInArea(x, y))
//        { //object alread here with some buffer, spawn in a different location
//            return this.getRandomCoordinatesInPlayableBounds();
//        }
        
        return {
            x: x,
            y: y,
        };
    },
                                      
    hasObjectInArea: function(x, y) {
        //janky way to make sure this isn't going to be on another item already on the map
        var buffer = 10;
        
        //check each item in sensitive groups
        var game = this;
        
        if (this.badMemoriesGroup)
        {
            this.badMemoriesGroup.forEach(function(s) {
                if (!game.spriteInArea(s, x, y, buffer))
                    return true;
            });
        }
        
        if (this.neuroTransmittorsGroup)
        {
            this.neuroTransmittorsGroup.forEach(function(s) {
                if (!game.spriteInArea(s, x, y, buffer))
                    return true;
            });
        }
        
        return false;
    },

    spriteInArea: function(sprite, x, y, buffer) {
        //this looks really shitty because scaling hasn't taken effect
        var center = {
            x: sprite.x + sprite.width / 2,
            y: sprite.y + sprite.height / 2,
        };
        
        var startX = center.x - buffer; //.center.x - buffer; //sprite.x - buffer;
        var endX = center.x + buffer; //sprite.x + sprite.width + buffer;
        var startY = center.y - buffer; //sprite.y - buffer;
        var endY = center.y + buffer; //sprite.height + buffer;
        
        //console.log(sprite);
        
        return ((x >= startX && x <= endX) && (y >= startY && y <= endY));
    },
    
    updateHud:function() {
        this.hud.progressBar.foregroundSprite.width = this.hud.progressBar.maxWidth * this.hud.progressBar.fillPercentage;
        this.hud.scoreText.text = "Good Thoughts Collected: " + this.hud.numberOfGoodMemories;
        
        var step = .01;
        if (this.hud.bloodOverlay.alpha > 0)
        {
            if (this.hud.bloodOverlay.alpha - step <= 0)
                this.hud.bloodOverlay.alpha = 0;
            else
                this.hud.bloodOverlay.alpha -= step;            
        }
        if (this.hud.healOverlay.alpha > 0)
        {
            if (this.hud.healOverlay.alpha - step <= 0)
                this.hud.healOverlay.alpha = 0;
            else
                this.hud.healOverlay.alpha -= step;            
        }
    },
    
    buildPlayer:function(){
        // Set up the player
        this.player = this.add.sprite(20, 50, 'dude');
        // Set anchor to allow for sprite mirroring
        this.player.anchor.setTo(0.5, 0.5);
        
        this.physics.p2.enable(this.player);
        this.player.body.setCollisionGroup(this.collisionGroups.player);
        this.player.smoothed = false;
        this.player.body.fixedRotation = true;

        // Create animations for player
        // this.player.animations.add('moonWalk', [0, 1, 2, 3], 10, true);           

        this.player.animations.add('walk', [1, 2, 3, 4, 5, 6, 7, 8, 9, 8, 7, 6, 5, 4, 3, 2], 15, true);  
        
        // Camera follow player
        this.camera.follow(this.player);
    },
    
    buildWorld:function() {
        
        this.platformGroup = this.add.group();
        this.platformGroup.enableBody = true;
        this.platformGroup.physicsBodyType = Phaser.Physics.P2JS;
        
        //some randomizing of the floor generation
        var playableBounds = this.getPlayableBounds();
        var numOfDifferentTiles = 1;
        var currentX = 0;
        var floorY = this.world.height - 120;
        var floorWidth = 800;
        var numFloors = (this.world.width + floorWidth) / floorWidth;
        var floorName = 'floor1';
        
        for (var i=0; i<numFloors; i++)
        {
            
            floor = this.platformGroup.create(i * floorWidth, floorY, floorName);
            floor.body.clearShapes();
            floor.body.loadPolygon('physics', floorName);
            if (this.rnd.integerInRange(0, 1) % 2 === 1)
            {
                floor.scale.setTo(-1, 1); //randomly have chance to flip horizontally
            }
            floor.body.static = true;
            floor.body.setCollisionGroup(this.collisionGroups.platforms);
            floor.body.collides(this.collisionGroups.player);
            
//            floor.body.debug = true;
        }
    },

};